package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.AdminOperateType;
import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDetailService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordWaitingAuditService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.UserDataRules;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamStudentQuestionScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 考试记录相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试记录相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/record")
public class ExamRecordController extends ControllerSupport {

    @Autowired
    private ExamRecordService examRecordService;

    @Autowired
    private ExamRecordDetailService examRecordDetailService;

    @Autowired
    private ExamRecordWaitingAuditService examRecordWaitingAuditService;

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/waiting/audit/list")
    @ApiOperation(value = "查询“监考待审”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordWaitingAuditList(@RequestBody ExamRecordQuery query) {
        UserDataRule courseRule = super.getUserDataRule(DataRuleType.COURSE);
        UserDataRule orgRule = super.getUserDataRule(DataRuleType.ORG);

        Page<ExamRecordInfo> page = examRecordWaitingAuditService.getExamRecordWaitingAuditList(query, courseRule, orgRule);

        page.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        return page;
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/detail/check")
    @ApiOperation(value = "查询“考试明细”是否存在待审数据")
    public Long existsWarnExamRecordDetail(@RequestBody ExamRecordQuery query) {
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        return examRecordService.existsWarnExamRecordDetail(uds, query);
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/detail/list")
    @ApiOperation(value = "查询“考试明细”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordDetailList(@RequestBody ExamRecordQuery query) {
        UserDataRule courseRule = super.getUserDataRule(DataRuleType.COURSE);
        UserDataRule orgRule = super.getUserDataRule(DataRuleType.ORG);

        Page<ExamRecordInfo> page = examRecordDetailService.getExamRecordDetailList(query, courseRule, orgRule);

        page.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        return page;
    }

    @GetMapping("/detail/list/export/async")
    @ApiOperation(value = "导出“考试明细”列表(异步)", notes = "参数示例：query={\"rootOrgId\":0,\"examId\":1}")
    public void exportExamRecordDetails(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamRecordDetails(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), AdminOperateType.TYPE17.getDesc(), "导出条件:" + query));
    }

    @GetMapping("/select/byExamStudentId")
    @ApiOperation(value = "根据考生ID查询考试记录")
    public List<ExamRecordDataEntity> getExamRecordByExamStudentId(@RequestParam Long examStudentId) {
        List<ExamRecordDataEntity> examRecordEntityList = examRecordDataRepo.findByExamStudentId(examStudentId);
        return examRecordEntityList;
    }

    @GetMapping("/detail/examStudentQuestionScoreList")
    @ApiOperation(value = "导出考生作答分数明细列表(Excel)", notes = "参数examId和examId必传")
    public void getExamStudentQuestionScoreList(@RequestParam Long examId, @RequestParam String courseCode,
                                                HttpServletResponse response) throws Exception {
        Check.isNull(examId, "考试id不能为空！");
        Check.isBlank(courseCode, "课程代码不能空！");
        List<ExamStudentQuestionScoreInfo> examRecordInfoList =
                examRecordService.getExamStudentQuestionScoreList(examId, courseCode);
        ExportUtils.exportEXCEL("考试明细列表", ExamStudentQuestionScoreInfo.class, examRecordInfoList, response);
    }

}
