package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.MarkingType;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.ExamStudentEffectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.UserDataRules;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamStudentQuestionScoreInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考试记录接口
 *
 * @author: QMTH
 * @since: 2018/8/16
 */
@Service
public class ExamRecordServiceImpl implements ExamRecordService {

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    /**
     * 考生作答成绩
     */
    private static final String EXAM_STUDENT_QUESTION_SCORE_SQL = "select " +
            "erd.id exam_record_data_id,co.`code` course_code,co.`name` course_name, ex.`name` exam_name" +
            ",erd.student_code,erd.base_paper_id " +
            "from ec_oe_exam_record_data erd " +
            "inner join ec_b_course co on erd.course_id=co.id " +
            "inner join ec_e_exam ex on erd.exam_id=ex.id " +
            "where 1=1 ";

    /**
     * 构建查询 考试明细sql语句
     *
     * @param query
     * @return
     */
    private StringBuilder buildExamRecordCommonSelectCondition(UserDataRules uds, ExamRecordQuery query) {
        StringBuilder sql = new StringBuilder();
        if (query.getSwitchScreenCountStart() != null) {
            sql.append(" and record_data.switch_screen_count >= " + query.getSwitchScreenCountStart());
        }
        if (query.getSwitchScreenCountEnd() != null) {
            sql.append(" and record_data.switch_screen_count <= " + query.getSwitchScreenCountEnd());
        }
        if (query.getExamRecordDataId() != null) {
            sql.append(" and record_data.id = " + query.getExamRecordDataId());
        }
        if (query.getOrgId() != null) {
            sql.append(" and record_data.org_id = " + query.getOrgId());
        }
        if (query.getExamId() != null) {
            sql.append(" and record_data.exam_id = " + query.getExamId());
        }
        if (query.getExamStageId() != null) {
            sql.append(" and record_data.exam_stage_id = " + query.getExamStageId());
        }

        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and record_data.student_code LIKE '" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and record_data.student_name LIKE '" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and record_data.identity_number LIKE '" + query.getIdentityNumber() + "%'");
        }
        if (StringUtils.isNotBlank(query.getInfoCollector())) {
            sql.append(" and record_data.info_collector LIKE '" + query.getInfoCollector() + "%'");
        }
        if (StringUtils.isNotBlank(query.getAuditUserName())) {
            sql.append(" and audit.audit_user_name LIKE '" + query.getAuditUserName() + "%'");
        }
        if (query.getCourseId() != null) {
            sql.append(" and record_data.course_id = " + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and record_data.course_level = '" + query.getCourseLevel() + "'");
        }
        if (query.getExamRecordId() != null) {
            sql.append(" and record_data.id = " + query.getExamRecordId());
        }
        if (query.getIsIllegality() != null) {
            if (query.getIsIllegality()) {
                sql.append(" and record_data.is_illegality = 1");
            } else {
                sql.append(" and record_data.is_illegality = 0");
            }
        }
        if (StringUtils.isNotBlank(query.getStartTime()) && StringUtils.isNotBlank(query.getEndTime())) {
            sql.append(" and record_data.start_time >= str_to_date('" + query.getStartTime() + "','%Y/%m/%d %H:%i:%s')"
                    + " and record_data.start_time <= str_to_date('" + query.getEndTime() + "','%Y/%m/%d %H:%i:%s')");
        }

        if (StringUtils.isNotBlank(query.getSubmitStartTime()) && StringUtils.isNotBlank(query.getSubmitEndTime())) {
            sql.append(" and record_data.end_time is not null and record_data.end_time >= str_to_date('" + query.getSubmitStartTime() + "','%Y/%m/%d %H:%i:%s')"
                    + " and record_data.end_time <= str_to_date('" + query.getSubmitEndTime() + "','%Y/%m/%d %H:%i:%s')");
        }

        //考试记录状态
        List<ExamRecordStatus> statusList = query.getRecordStatusList();
        if (statusList != null && statusList.size() > 0) {
            String status = "";
            for (int i = 0; i < statusList.size(); i++) {
                status += "'" + statusList.get(i) + "',";
            }
            status = status.substring(0, status.lastIndexOf(","));
            sql.append(" and record_data.exam_record_status IN (" + status + ")");
        }
        //是否有陌生人脸
        if (query.getHasStranger() != null) {
            if (query.getHasStranger()) {
                sql.append(" and record_data.face_stranger_count > 0");
            } else {
                sql.append(" and record_data.face_stranger_count = 0");
            }
        }
        //人脸识别成功率
        if (query.getFaceSuccessPercentLower() != null && query.getFaceSuccessPercentUpper() != null) {
            sql.append(" and record_data.face_success_percent >=" + query.getFaceSuccessPercentLower());
            sql.append(" and record_data.face_success_percent <=" + query.getFaceSuccessPercentUpper());
        }
        if (query.getLivenessSuccessPercentLower() != null && query.getLivenessSuccessPercentUpper() != null) {
            sql.append(" and record_data.baidu_face_liveness_success_percent >=" + query.getLivenessSuccessPercentLower());
            sql.append(" and record_data.baidu_face_liveness_success_percent <=" + query.getLivenessSuccessPercentUpper());
        }

        //是否有虚拟设备
        if (query.getHasVirtual() != null) {
            if (query.getHasVirtual()) {
                sql.append(" AND EXISTS( ");
                sql.append(" select 1 from ec_oe_exam_capture_camera_info v where record_data.id = v.exam_record_data_id and v.virtual_camera=1 ");
                sql.append(" ) ");
            } else {
                sql.append(" AND NOT EXISTS( ");
                sql.append(" select 1 from ec_oe_exam_capture_camera_info v where record_data.id = v.exam_record_data_id and v.virtual_camera=1 ");
                sql.append(" ) ");
            }
        }
        if (StringUtils.isNotBlank(query.getVirtualName())) {
            sql.append(" AND EXISTS( ");
            sql.append(" select 1 from ec_oe_exam_capture_camera_info ci where record_data.id = ci.exam_record_data_id ");
            sql.append(" and ci.virtual_camera=1 and ci.name='" + query.getVirtualName() + "' ");
            sql.append(" ) ");
        }

        //ip
        if (StringUtils.isNotBlank(query.getIp())) {
            sql.append(" and process.source_ip='" + query.getIp() + "' ");
        }
        if (uds.getOrgRule().assertNeedQueryRefIds()) {
            sql.append(" and record_data.org_id in (" + StringUtils.join(uds.getOrgRule().getRefIds(), ",") + ") ");
        }
        if (uds.getCourseRule().assertNeedQueryRefIds()) {
            sql.append(" and record_data.course_id in (" + StringUtils.join(uds.getCourseRule().getRefIds(), ",") + ") ");
        }
        return sql;
    }

    @Override
    public List<ExamRecordDataEntity> getExamRecordListByExamStudentId(Long examStudentId) {
        Check.isNull(examStudentId, "考生ID不能为空！");
        return examRecordDataRepo.findByExamStudentId(examStudentId);
    }

    @Override
    public Long existsWarnExamRecordDetail(UserDataRules uds, ExamRecordQuery query) {
        if (uds.getCourseRule().assertEmptyQueryResult() || uds.getOrgRule().assertEmptyQueryResult()) {
            return 0L;
        }
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("select count(record_data.id) from ec_oe_exam_record_data record_data "
                + " where 1=1 ");
        sqlBuilder.append(buildExamRecordCommonSelectCondition(uds, query));
        sqlBuilder.append(" and record_data.is_warn = 1 and record_data.is_audit = 0");
        return jdbcTemplate.queryForObject(sqlBuilder.toString(), Long.class);
    }

    @Override
    public List<ExamStudentQuestionScoreInfo> getExamStudentQuestionScoreList(Long examId, String courseCode) {
        //最终返回的结果集
        List<ExamStudentQuestionScoreInfo> resultList = new ArrayList<ExamStudentQuestionScoreInfo>();
        //获取考试记录相关基本信息
        List<ExamStudentQuestionScoreInfo> partialExamStudentQuestionScoreList = getPartialExamStudentQuestionScoreList(examId, courseCode);
        List<Long> examRecordDataIdList = new ArrayList<Long>();
        if (partialExamStudentQuestionScoreList != null && !partialExamStudentQuestionScoreList.isEmpty()) {
            partialExamStudentQuestionScoreList.forEach(p -> examRecordDataIdList.add(p.getExamRecordDataId()));
            //批量查询考试记录数据
            List<ExamRecordQuestionsEntity> examRecordQuestionsList = examRecordQuestionsRepo.findByExamRecordDataIdIn(examRecordDataIdList);
            for (ExamStudentQuestionScoreInfo scoreInfo : partialExamStudentQuestionScoreList) {
                List<ExamRecordQuestionsEntity> filterExamRecordQuestionsList = examRecordQuestionsList.stream().filter(p -> p.getExamRecordDataId().equals(scoreInfo.getExamRecordDataId())).collect(Collectors.toList());
                if (examRecordQuestionsList != null && !examRecordQuestionsList.isEmpty()) {
                    ExamRecordQuestionsEntity filterExamRecordQuestionsEntity = filterExamRecordQuestionsList.get(0);
                    //当前考试记录中的所有考题成绩集合
                    List<ExamQuestionEntity> examQuestionEntities = filterExamRecordQuestionsEntity.getExamQuestionEntities();
                    if (examQuestionEntities != null && !examQuestionEntities.isEmpty()) {
                        examQuestionEntities.forEach(qa -> {
                            ExamStudentQuestionScoreInfo examStudentQuestionScoreInfo = buildExamStudentQuestionScore(scoreInfo, qa);
                            resultList.add(examStudentQuestionScoreInfo);
                        });
                    }
                }
            }
        }

        return resultList;
    }

    @Override
    public List<ExamStudentEffectiveScoreInfo> getExamStudentEffectiveScoreList(String markingType, List<Long> examStudentIdList) {
        List<ExamStudentEffectiveScoreInfo> resultList = new ArrayList<>();
        //考生所有的考试记录
        List<ExamRecordDataEntity> allExamRecordDataList = examRecordDataRepo.findEffectiveDataByExamStudentIdList(examStudentIdList);
        if (allExamRecordDataList == null || allExamRecordDataList.isEmpty()) {
            return null;
        }
        List<Long> examRecordDataIdList = allExamRecordDataList.stream().map(p -> p.getId()).collect(Collectors.toList());
        //当前考试记录的所有成绩
        List<ExamScoreEntity> allExamScoreList = examScoreRepo.findByExamRecordDataIdIn(examRecordDataIdList);
        if (allExamScoreList == null || allExamScoreList.isEmpty()) {
            return null;
        }
        Map<Long, List<ExamRecordDataEntity>> groupedByExamStudentExamRecordMap = allExamRecordDataList.stream().collect(Collectors.groupingBy(ExamRecordDataEntity::getExamStudentId));
        for (Long examStudentId : groupedByExamStudentExamRecordMap.keySet()) {
            //当前考生的所有考试记录集合
            List<ExamRecordDataEntity> curExamRecordDataList = groupedByExamStudentExamRecordMap.get(examStudentId);
            //当前考试记录id集合
            List<Long> curExamRecordDataIdList = curExamRecordDataList.stream().map(p -> p.getId()).collect(Collectors.toList());

            //当前考生的所有考试成绩集合
            List<ExamScoreEntity> curExamScoreList = allExamScoreList.stream().
                    filter(p -> curExamRecordDataIdList.contains(p.getExamRecordDataId())).collect(Collectors.toList());

            //最终生效的成绩
            ExamScoreEntity effectiveScore = getStudentEffectiveScore(curExamScoreList, markingType);
            //最终生效的考试记录
            ExamRecordDataEntity effectiveExamRecordData = curExamRecordDataList.stream().
                    filter(p -> p.getId().equals(effectiveScore.getExamRecordDataId())).collect(Collectors.toList()).get(0);

            ExamStudentEffectiveScoreInfo info = new ExamStudentEffectiveScoreInfo();
            info.setExamStudentId(examStudentId);
            info.setRootOrgId(effectiveExamRecordData.getRootOrgId());
            info.setOrgId(effectiveExamRecordData.getOrgId());
            info.setBasePaperId(effectiveExamRecordData.getBasePaperId());
            info.setCourseId(effectiveExamRecordData.getCourseId());
            info.setScore(effectiveScore.getTotalScore());
            info.setExamRecordDataId(effectiveExamRecordData.getId());
            resultList.add(info);
        }

        return resultList;
    }

    /**
     * @Description 获取考生一次考试的最终有效成绩
     * @Author lideyin
     * @Date 2019/7/18 17:49
     * @Version 1.0
     */
    private ExamScoreEntity getStudentEffectiveScore(List<ExamScoreEntity> curExamScoreList, String markingType) {
        if (curExamScoreList == null || curExamScoreList.isEmpty()) {
            throw new StatusException("201101", "考试记录不为空，考试分数数据为空");
        }
        ExamScoreEntity examScoreEntity;
        //阅卷方式为全部评阅或客观题分数最高，均取总分最高
        if (MarkingType.ALL.toString().equals(markingType) || MarkingType.OBJECT_SCORE_MAX.toString().equals(markingType)) {
            examScoreEntity = curExamScoreList.stream().max(Comparator.comparing(p -> p.getTotalScore())).get();
        } else if (MarkingType.LAST_SUBMIT.toString().equals(markingType)) {
            examScoreEntity = curExamScoreList.stream().max(Comparator.comparing(p -> p.getId())).get();
        } else {
            throw new StatusException("201001", "阅卷类型不支持");
        }
        return examScoreEntity;
    }

    /**
     * 根据考试记录信息和考题信息构建完整 的考生考题作答记录
     *
     * @param scoreInfo
     * @param qa
     * @return
     */
    private ExamStudentQuestionScoreInfo buildExamStudentQuestionScore(ExamStudentQuestionScoreInfo scoreInfo,
                                                                       ExamQuestionEntity qa) {
        ExamStudentQuestionScoreInfo examStudentQuestionScoreInfo = new ExamStudentQuestionScoreInfo();
        examStudentQuestionScoreInfo.setExamRecordDataId(scoreInfo.getExamRecordDataId());
        examStudentQuestionScoreInfo.setCourseCode(scoreInfo.getCourseCode());
        examStudentQuestionScoreInfo.setCourseName(scoreInfo.getCourseName());
        examStudentQuestionScoreInfo.setExamName(scoreInfo.getExamName());
        examStudentQuestionScoreInfo.setStudentCode(scoreInfo.getStudentCode());
        examStudentQuestionScoreInfo.setBasePaperId(scoreInfo.getBasePaperId());
        examStudentQuestionScoreInfo.setMainNumber(qa.getMainNumber());
        examStudentQuestionScoreInfo.setOrder(qa.getOrder());
        examStudentQuestionScoreInfo.setStudentScore(qa.getStudentScore());
        examStudentQuestionScoreInfo.setStudentAnswer(qa.getStudentAnswer());

        // if (QuestionType.SINGLE_CHOICE == qa.getQuestionType() || QuestionType.MULTIPLE_CHOICE == qa.getQuestionType()) {
        //     // 选择题 - 转换为新答案格式（兼容旧格式）
        //     examStudentQuestionScoreInfo.setStudentAnswer(QuestionOptionHelper.parseNumbers(qa.getStudentAnswer()));
        // } else {
        //     examStudentQuestionScoreInfo.setStudentAnswer(qa.getStudentAnswer());
        // }

        return examStudentQuestionScoreInfo;
    }

    /**
     * 根据考试id和课程代码获取学生考试成绩的部分信息
     *
     * @param examId     考试id
     * @param courseCode 课程代码
     * @return
     */
    private List<ExamStudentQuestionScoreInfo> getPartialExamStudentQuestionScoreList(Long examId, String courseCode) {
        //查询条件
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append(EXAM_STUDENT_QUESTION_SCORE_SQL);
        sqlBuilder.append(" and erd.exam_id=" + examId);
        sqlBuilder.append(" and co.`code`='" + courseCode + "'");
        List<ExamStudentQuestionScoreInfo> examRecordDataList = jdbcTemplate.query(sqlBuilder.toString(), new RowMapper<ExamStudentQuestionScoreInfo>() {
            @Override
            public ExamStudentQuestionScoreInfo mapRow(ResultSet rs, int rowNum) throws SQLException {
                return getExamStudentQuestionScoreInfoByResultSet(rs);
            }
        });
        return examRecordDataList;
    }

    /**
     * 根据结果集获取相应的实体对象
     *
     * @param rs
     * @return
     * @throws SQLException
     */
    private ExamStudentQuestionScoreInfo getExamStudentQuestionScoreInfoByResultSet(ResultSet rs) throws SQLException {
        ExamStudentQuestionScoreInfo info = new ExamStudentQuestionScoreInfo();
        info.setExamRecordDataId(rs.getLong("exam_record_data_id"));
        info.setCourseCode(rs.getString("course_code"));
        info.setCourseName(rs.getString("course_name"));
        info.setExamName(rs.getString("exam_name"));
        info.setStudentCode(rs.getString("student_code"));
        info.setBasePaperId(rs.getString("base_paper_id"));
        return info;
    }

}
